"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const { keys, pick, omit, flatten, difference, extend } = require('lodash');
const EventTypes_1 = require("./EventTypes/EventTypes");
const EventStoreFactory_1 = require("./Factory/EventStoreFactory");
const Persistence_1 = require("./Persistence/Persistence");
const _ = require("lodash");
const flux_standard_action_1 = require("flux-standard-action");
let DEBUG = true;
class EventStore {
    constructor(framework) {
        this.framework = framework;
        this.EventTypes = EventTypes_1.EventTypes;
        this.EventStoreFactory = EventStoreFactory_1.EventStoreFactory;
        this.Persistence = Persistence_1.Persistence;
        this.writeEsEvent = (eventStore, fromAddress, esEvent) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let { Type, Version, ValueType, IsAuthorized, PermissionDomain, AddressValue, UIntValue, Bytes32Value, StringValue } = esEvent;
            return yield eventStore.writeEvent(Type, Version, ValueType, IsAuthorized, PermissionDomain, AddressValue, UIntValue, Bytes32Value, StringValue, {
                from: fromAddress,
                gas: 2000000
            });
        });
        this.readEsEventValues = (eventStore, fromAddress, eventId) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield eventStore.readEvent.call(eventId, {
                from: fromAddress,
                gas: 2000000
            });
        });
        this.readTransmuteEvent = (eventStore, fromAddress, eventId) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let eventVals = yield this.readEsEventValues(eventStore, fromAddress, eventId);
            let esEventWithTruffleTypes = EventTypes_1.EventTypes.getEsEventFromEsEventValues(eventVals);
            let esEvent = EventTypes_1.EventTypes.getEsEventFromEsEventWithTruffleTypes('EsEvent', esEventWithTruffleTypes);
            let transmuteEvent = yield EventTypes_1.EventTypes.esEventToTransmuteEvent(esEvent);
            if (typeof transmuteEvent.payload === 'string' && transmuteEvent.payload.indexOf('ipfs/') !== -1) {
                if (!this.framework.TransmuteIpfs.ipfs) {
                    this.framework.TransmuteIpfs.init({
                        host: 'localhost',
                        port: '5001',
                        options: {
                            protocol: 'http'
                        }
                    });
                }
                let path = transmuteEvent.payload;
                transmuteEvent.payload = yield this.framework.TransmuteIpfs.readObject(path);
                transmuteEvent.meta.path = path;
            }
            if (DEBUG && !flux_standard_action_1.isFSA(transmuteEvent)) {
                console.warn('WARNING: transmuteEvent: ', transmuteEvent, ' is not a FSA. see https://github.com/acdlite/flux-standard-action');
            }
            return transmuteEvent;
        });
        this.readTransmuteEvents = (eventStore, fromAddress, eventId = 0) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let currentEvent = (yield eventStore.solidityEventCount()).toNumber();
            let eventPromises = [];
            while (eventId < currentEvent) {
                eventPromises.push(yield this.readTransmuteEvent(eventStore, fromAddress, eventId));
                eventId++;
            }
            return yield Promise.all(eventPromises);
        });
        this.writeTransmuteCommand = (eventStore, fromAddress, transmuteCommand) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let esEvent = EventTypes_1.EventTypes.convertCommandToEsEvent(transmuteCommand);
            let isIpfsObject = typeof transmuteCommand.payload === 'object';
            let hash;
            if (isIpfsObject) {
                hash = yield this.framework.TransmuteIpfs.writeObject(transmuteCommand.payload);
                esEvent.ValueType = 'String';
                esEvent.StringValue = 'ipfs/' + hash;
            }
            let tx = yield this.writeEsEvent(eventStore, fromAddress, esEvent);
            let eventsFromWriteEsEvent = yield EventTypes_1.EventTypes.eventsFromTransaction(tx);
            let esEventWithIndex = eventsFromWriteEsEvent[0];
            let allTxs = [tx];
            allTxs = _.flatten(allTxs);
            let transmuteEvents = yield Promise.all(EventTypes_1.EventTypes.reconstructTransmuteEventsFromTxs(allTxs));
            if (isIpfsObject) {
                transmuteEvents[0].payload = transmuteCommand.payload;
                transmuteEvents[0].meta.path = 'ipfs/' + hash;
            }
            return {
                events: transmuteEvents,
                transactions: allTxs
            };
        });
        this.writeTransmuteCommands = (eventStore, fromAddress, transmuteCommands) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let promises = transmuteCommands.map((cmd) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                return yield this.writeTransmuteCommand(eventStore, fromAddress, cmd);
            }));
            return yield Promise.all(promises);
        });
        this.readModelGenerator = (readModel, reducer, events) => {
            events.forEach((event) => {
                readModel = reducer(readModel, event);
            });
            return readModel;
        };
        this.maybeSyncReadModel = (eventStore, fromAddress, readModel, reducer) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            let solidityEventCount = (yield eventStore.solidityEventCount()).toNumber();
            return this.framework.Persistence.LocalStore.getItem(readModel.readModelStoreKey)
                .then((_readModel) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                if (!_readModel) {
                    _readModel = readModel;
                }
                if (_readModel.lastEvent === solidityEventCount) {
                    return readModel;
                }
                let startIndex = _readModel.lastEvent !== null ? _readModel.lastEvent + 1 : 0;
                let events = yield this.readTransmuteEvents(eventStore, fromAddress, startIndex);
                let updatedReadModel = this.readModelGenerator(_readModel, reducer, events);
                return this.framework.Persistence.LocalStore.setItem(updatedReadModel.readModelStoreKey, updatedReadModel);
            }));
        });
        this.getCachedReadModel = (contractAddress, eventStore, fromAddress, readModel, reducer) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            readModel.readModelStoreKey = `${readModel.readModelType}:${contractAddress}`;
            readModel.contractAddress = contractAddress;
            readModel = yield this.maybeSyncReadModel(eventStore, fromAddress, readModel, reducer);
            return readModel;
        });
    }
}
exports.EventStore = EventStore;
