"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
var ethUtils = require('ethereumjs-util');
const camelcaseKeys = require('camelcase-keys');
const _ = require("lodash");
const TestEvents_1 = require("../Mock/Events/TestEvents");
var EventTypes;
(function (EventTypes) {
    EventTypes.toAscii = (value) => {
        return ethUtils.toAscii(value).replace(/\u0000/g, '');
    };
    EventTypes.guessType = (value) => {
        if (typeof value === 'number') {
            return 'UInt';
        }
        if (typeof value === 'object') {
            return 'String';
        }
        if (typeof value === 'string') {
            if (ethUtils.isValidAddress(value)) {
                return 'Address';
            }
            return 'String';
        }
        throw Error('unable to guess type of value: ' + value);
    };
    EventTypes.EsEvent = 'EsEvent';
    EventTypes.EsEventSchema = {
        Id: 'BigNumber',
        Type: 'Bytes32',
        Version: 'Bytes32',
        ValueType: 'Bytes32',
        IsAuthorized: 'Boolean',
        PermissionDomain: 'Bytes32',
        AddressValue: 'String',
        UIntValue: 'BigNumber',
        Bytes32Value: 'Bytes32',
        StringValue: 'String',
        TxOrigin: 'String',
        Created: 'BigNumber'
    };
    EventTypes.TruffleEventSchema = {
        [EventTypes.EsEvent]: EventTypes.EsEventSchema
    };
    EventTypes.getPropFromTruffleEventSchemaType = (key, schema, truffleValue) => {
        let schemaPropType = schema[key];
        switch (schemaPropType) {
            case 'String': return truffleValue.toString();
            case 'Address': return truffleValue.toString();
            case 'String': return truffleValue.toString();
            case 'Bytes32': return EventTypes.toAscii(truffleValue);
            case 'BigNumber': return truffleValue.toNumber();
            case 'UInt': return truffleValue.toNumber();
            case 'Boolean': return truffleValue;
            default: throw Error(`UNKNOWN schemaPropType for key: '${key} for truffleValue '${truffleValue}'. Make sure your schema is up to date and contains all keys`);
        }
    };
    EventTypes.getEsEventFromEsEventValues = (eventValues) => {
        let evt = {};
        _.keys(EventTypes.EsEventSchema).map((k, i) => {
            evt[k] = eventValues[i];
        });
        return evt;
    };
    EventTypes.getEsEventFromEsEventWithTruffleTypes = (eventType, solEvent) => {
        let event = {};
        let schema = EventTypes.TruffleEventSchema[eventType];
        _.forIn(solEvent, (value, key) => {
            let prop = EventTypes.getPropFromTruffleEventSchemaType(key, schema, value);
            _.extend(event, {
                [key]: prop
            });
        });
        return event;
    };
    EventTypes.getValueType = (es) => {
        switch (es.ValueType) {
            case 'Address': return es.AddressValue;
            case 'String': return es.StringValue;
            case 'UInt': return es.UIntValue;
            case 'Bytes32': return es.Bytes32Value;
            default: throw Error('ValueType invalid, must be [UInt, Address, Bytes32, String]');
        }
    };
    EventTypes.convertMeta = (esEvent) => {
        let metaKeys = ['Type', 'ValueType', 'IsAuthorized', 'PermissionDomain', 'AddressValue', 'UIntValue', 'Bytes32Value', 'StringValue'];
        let objectWithoutEsMeta = _.omit(esEvent, metaKeys);
        let withProperCase = camelcaseKeys(objectWithoutEsMeta);
        return withProperCase;
    };
    EventTypes.esEventToTransmuteEvent = (esEvent) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        let payload = {};
        let meta = EventTypes.convertMeta(esEvent);
        payload = EventTypes.getValueType(esEvent);
        return {
            type: esEvent.Type,
            payload: payload,
            meta: meta
        };
    });
    EventTypes.convertCommandToEsEvent = (transmuteCommand) => {
        let valueType = EventTypes.guessType(transmuteCommand.payload);
        let esEvent;
        if (valueType === 'UInt' && transmuteCommand.payload < 0) {
            throw Error('Only unsigned integers are supported for number payloads. consider using a string.');
        }
        switch (valueType) {
            case 'Address':
                esEvent = _.cloneDeep(TestEvents_1.addressValueEsEvent);
                esEvent.AddressValue = transmuteCommand.payload;
                break;
            case 'UInt':
                esEvent = _.cloneDeep(TestEvents_1.uIntValueEsEvent);
                esEvent.UIntValue = transmuteCommand.payload;
                break;
            case 'Bytes32':
                esEvent = _.cloneDeep(TestEvents_1.bytes32ValueEsEvent);
                esEvent.Bytes32Value = transmuteCommand.payload;
                break;
            case 'String':
                esEvent = _.cloneDeep(TestEvents_1.stringValueEsEvent);
                esEvent.StringValue = transmuteCommand.payload;
                break;
            case 'Boolean':
                esEvent = _.cloneDeep(TestEvents_1.stringValueEsEvent);
                esEvent.StringValue = transmuteCommand.payload;
                break;
            default:
                throw Error('Unkown valueType: ' + valueType);
        }
        esEvent.Type = transmuteCommand.type;
        return esEvent;
    };
    EventTypes.flatten = (data) => {
        var result = {};
        function recurse(cur, prop) {
            if (Object(cur) !== cur) {
                result[prop] = cur;
            }
            else if (Array.isArray(cur)) {
                for (var i = 0, l = cur.length; i < l; i++)
                    recurse(cur[i], prop + "[" + i + "]");
                if (l == 0)
                    result[prop] = [];
            }
            else {
                var isEmpty = true;
                for (var p in cur) {
                    isEmpty = false;
                    recurse(cur[p], prop ? prop + "." + p : p);
                }
                if (isEmpty && prop)
                    result[prop] = {};
            }
        }
        recurse(data, "");
        return result;
    };
    EventTypes.unflatten = (data) => {
        if (Object(data) !== data || Array.isArray(data))
            return data;
        var regex = /\.?([^.\[\]]+)|\[(\d+)\]/g, resultholder = {};
        for (var p in data) {
            var cur = resultholder, prop = "", m;
            while (m = regex.exec(p)) {
                cur = cur[prop] || (cur[prop] = (m[2] ? [] : {}));
                prop = m[2] || m[1];
            }
            cur[prop] = data[p];
        }
        return resultholder[""] || resultholder;
    };
    EventTypes.eventsFromTransaction = (tx) => {
        let allEvents = tx.logs.map((log) => {
            return EventTypes.getEsEventFromEsEventWithTruffleTypes(log.event, log.args);
        });
        return allEvents;
    };
    EventTypes.reconstructTransmuteEventsFromTxs = (txs) => {
        let esEventsAndEventProps = _.flatten(txs.map(EventTypes.eventsFromTransaction));
        let esEvents = _.filter(esEventsAndEventProps, (obj) => {
            return obj.Id !== undefined;
        });
        let transmuteEvents = [];
        esEvents.forEach((esEvent) => {
            let transmuteEvent = EventTypes.esEventToTransmuteEvent(esEvent);
            transmuteEvents.push(transmuteEvent);
        });
        return transmuteEvents;
    };
})(EventTypes = exports.EventTypes || (exports.EventTypes = {}));
